unit CorProf;

//Translated from corprof.h

{.$define REALLY_NAKED}

interface

uses
  ActiveX, Windows;

type
  PPByte = ^PByte;
  PULONGArray = ^TULONGArray;
  TULONGArray = array[0..16387] of ULONG;

(* ICorDebugCode:: GetILToNativeMapping returns an array of
 * COR_DEBUG_IL_TO_NATIVE_MAP structures.  In order to convey that certain
 * ranges of native instructions correspond to special regions of code (for
 * example, the prolog), an entry in the array may have it's ilOffset field set
 * to one of these values.
 *)
type
  CorDebugIlToNativeMappingTypes = TOleEnum;
const
  NO_MAPPING = $FFFFFFFF;
  PROLOG = $FFFFFFFE;
  EPILOG = $FFFFFFFD;

// Constants for enum COR_PRF_MONITOR
type
  COR_PRF_MONITOR = TOleEnum;
const
  // These flags represent classes of callback events
  COR_PRF_MONITOR_NONE             = $00000000;
  COR_PRF_MONITOR_FUNCTION_UNLOADS = $00000001;
  COR_PRF_MONITOR_CLASS_LOADS      = $00000002;
  COR_PRF_MONITOR_MODULE_LOADS     = $00000004;
  COR_PRF_MONITOR_ASSEMBLY_LOADS   = $00000008;
  COR_PRF_MONITOR_APPDOMAIN_LOADS  = $00000010;
  COR_PRF_MONITOR_JIT_COMPILATION  = $00000020;
  COR_PRF_MONITOR_EXCEPTIONS       = $00000040;
  COR_PRF_MONITOR_GC               = $00000080;
  COR_PRF_MONITOR_OBJECT_ALLOCATED = $00000100;
  COR_PRF_MONITOR_THREADS          = $00000200;
  COR_PRF_MONITOR_REMOTING         = $00000400;
  COR_PRF_MONITOR_CODE_TRANSITIONS = $00000800;
  COR_PRF_MONITOR_ENTERLEAVE       = $00001000;
  COR_PRF_MONITOR_CCW              = $00002000;
  COR_PRF_MONITOR_REMOTING_COOKIE  = $00004000 or COR_PRF_MONITOR_REMOTING;
  COR_PRF_MONITOR_REMOTING_ASYNC   = $00008000 or COR_PRF_MONITOR_REMOTING;
  COR_PRF_MONITOR_SUSPENDS         = $00010000;
  COR_PRF_MONITOR_CACHE_SEARCHES   = $00020000;
  COR_PRF_MONITOR_CLR_EXCEPTIONS   = $01000000;

  // All callback events are enabled with this flag
  COR_PRF_MONITOR_ALL              = $0107FFFF;

  // This flag tells the runtime to allow the re-jitting of methods
  // (through calls to ICorProfilerInfo::SetFunctionReJIT)
  COR_PRF_ENABLE_REJIT             = $00040000;

  // This flag enables the use of the in-process debugging services
  COR_PRF_ENABLE_INPROC_DEBUGGING  = $00080000;

  // This tells the runtime that it should maintain Native to IL maps
  COR_PRF_ENABLE_JIT_MAPS          = $00100000;

  // This tells the runtime to disable all inlining
  COR_PRF_DISABLE_INLINING         = $00200000;

  // This tells the runtime to disable all code optimizations
  COR_PRF_DISABLE_OPTIMIZATIONS    = $00400000;

  // This tells the runtime that the profiler may want object allocation
  // notifications.  This must be set during initialization if the profiler
  // ever wants object notifications (using COR_PRF_MONITOR_OBJECT_ALLOCATED)
  COR_PRF_ENABLE_OBJECT_ALLOCATED  = $00800000;

  // The mask for valid mask bits
  COR_PRF_ALL                      = $01FFFFFF;

  // This represents all flags that may only be set during initialisation.
  // Trying to change any of these flags elsewhere will result in a
  // failed HRESULT.
  COR_PRF_MONITOR_IMMUTABLE = COR_PRF_MONITOR_CODE_TRANSITIONS or
                              COR_PRF_MONITOR_REMOTING or
                              COR_PRF_MONITOR_REMOTING_COOKIE or
                              COR_PRF_MONITOR_REMOTING_ASYNC or
                              COR_PRF_MONITOR_GC or
                              COR_PRF_ENABLE_REJIT or
                              COR_PRF_ENABLE_INPROC_DEBUGGING or
                              COR_PRF_ENABLE_JIT_MAPS or
                              COR_PRF_DISABLE_OPTIMIZATIONS or
                              COR_PRF_DISABLE_INLINING or
                              COR_PRF_ENABLE_OBJECT_ALLOCATED;

// Constants for enum COR_PRF_MISC
type
  COR_PRF_MISC = TOleEnum;
const
  PROFILER_PARENT_UNKNOWN = $FFFFFFFD;
  PROFILER_GLOBAL_CLASS = $FFFFFFFE;
  PROFILER_GLOBAL_MODULE = $FFFFFFFF;

// Constants for enum COR_PRF_JIT_CACHE
type
  COR_PRF_JIT_CACHE = TOleEnum;
const
  COR_PRF_CACHED_FUNCTION_FOUND = $00000000;
  COR_PRF_CACHED_FUNCTION_NOT_FOUND = $00000001;

// Constants for enum COR_PRF_TRANSITION_REASON
type
  COR_PRF_TRANSITION_REASON = TOleEnum;
const
  COR_PRF_TRANSITION_CALL = $00000000;
  COR_PRF_TRANSITION_RETURN = $00000001;

// Constants for enum COR_PRF_SUSPEND_REASON
type
  COR_PRF_SUSPEND_REASON = TOleEnum;
const
  COR_PRF_SUSPEND_OTHER = $00000000;
  COR_PRF_SUSPEND_FOR_GC = $00000001;
  COR_PRF_SUSPEND_FOR_APPDOMAIN_SHUTDOWN = $00000002;
  COR_PRF_SUSPEND_FOR_CODE_PITCHING = $00000003;
  COR_PRF_SUSPEND_FOR_SHUTDOWN = $00000004;
  COR_PRF_SUSPEND_FOR_INPROC_DEBUGGER = $00000006;
  COR_PRF_SUSPEND_FOR_GC_PREP = $00000007;

type
  ICorProfilerCallback = interface;
  ICorProfilerInfo = interface;
  IMethodMalloc = interface;

  COR_IL_MAP = packed record
    oldOffset: ULONG;
    newOffset: ULONG;
    fAccurate: Integer;
  end;

  COR_DEBUG_IL_TO_NATIVE_MAP = packed record
    ilOffset: ULONG;
    nativeStartOffset: ULONG;
    nativeEndOffset: ULONG;
  end;
  PCOR_DEBUG_IL_TO_NATIVE_MAP = ^COR_DEBUG_IL_TO_NATIVE_MAP;

  FunctionIDMapper = function(funcID: ULONG; var pbHookFunction: Bool): ULONG; stdcall;

(*
 * NOTE!!!
 *
 * It is VERY IMPORTANT to note that these function implementations must be
 * __declspec(naked), since the EE is not saving any registers before calling
 * any of them.  YOU MUST SAVE ALL REGISTERS YOU USE, INCLUDING FPU REGISTERS
 * IF THE FPU STACK IS NOT EMPTY AND YOU INTEND TO USE IT.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 *)
{$ifndef REALLY_NAKED}
  FunctionEnter = procedure(functionID: ULONG); stdcall;
  FunctionLeave = procedure(functionID: ULONG); stdcall;
  FunctionTailcall = procedure(functionID: ULONG); stdcall;
{$else}
  FunctionEnter = procedure;
  FunctionLeave = procedure;
  FunctionTailcall = procedure;
{$endif}

(* -------------------------------------------------------------------------- *
 * User Callback interface
 * -------------------------------------------------------------------------- *)

(*
 * The ICorProfilerCallback interface is used by the CLR to notify a
 * code profiler when events have occurred that the code profiler has registered
 * an in interest in receiving. This is the primary callback interface through
 * which the CLR communicates with the code profiler. A code profiler
 * must register this callback interface in the Win32 registry. This object has
 * several methods that receive notification from the runtime when an event is
 * about to occur in an executing runtime process.
 *
 * The methods implemented on this interface return S_OK on success, or E_FAIL
 * on failure.
 *)
  ICorProfilerCallback = interface(IUnknown)
    ['{176FBED1-A55C-4796-98CA-A9DA0EF883E7}']

    (*
     *
     * STARTUP/SHUTDOWN EVENTS
     *
     *)

    (*
     * The CLR calls Initialize to setup the code profiler
     * whenever a new CLR application is started. The call provides
     * an IUnknown interface pointer that should be QI'd for an ICorProfilerInfo
     * interface pointer.
     *
     * NOTE: this is the only opportunity to enable callbacks that are a part
     * of COR_PRF_MONITOR_IMMUTABLE, since they can no longer be changed after
     * returning from this function.  This is done through SetEventMask on the
     * ICorProfilerInfo object.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure Initialize(const pICorProfilerInfoUnk: IUnknown); safecall;

    (*
     * The CLR calls Shutdown to notify the code profiler that
     * the application is exiting.  This is the profiler's last opportunity to
     * safely call functions on the ICorProfilerInfo interface.  After returning
     * from this function the runtime will proceed to unravel its internal data
     * structures and any calls to ICorProfilerInfo are undefined in their
     * behaviour.
     *
     * NOTE: to maintain overall API speed, the profiler omits many argument
     * checks.  This means that the profiler is responsible for making sure
     * that the arguments it provides to any ICorProfilerInfo API are valid.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure Shutdown; safecall;

    (*
     *
     * APPLICATION DOMAIN EVENTS
     *
     *)

    (*
     * Called when an application domain creation has begun and ended.
     * The id is not valid for any information request until after the app
     * domain has been fully created.  One may only cache the id provided in
     * AppDomainCreationStarted for later use.
     * The hrStatus provides the success or failure of the operation
     *)
    procedure AppDomainCreationStarted(appDomainId: ULONG); safecall;
    procedure AppDomainCreationFinished(appDomainId: ULONG; hrStatus: HResult); safecall;

    (*
     * Called before and after an app domain is unloaded from a process.
     * Once an AppDomain unload has started, its ID is no longer valid in the
     * system and cannot be used in calls to any ICorProfilerInfo APIs.
     * The hrStatus provides the success or failure of the operation
     *)
    procedure AppDomainShutdownStarted(appDomainId: ULONG); safecall;
    procedure AppDomainShutdownFinished(appDomainId: ULONG; hrStatus: HResult); safecall;

    (*
     *
     * ASSEMBLY EVENTS
     *
     *)

    (*
     * Called when an Assembly load has begun and ended. The id is not valid
     * for any information request until after the assembly has been fully
     * loaded.  One may only cache the id provided in AssemblyLoadStarted for
     * later use.
     * The hrStatus provides the success or failure of the operation
     *)
    procedure AssemblyLoadStarted(assemblyId: ULONG); safecall;
    procedure AssemblyLoadFinished(assemblyId: ULONG; hrStatus: HResult); safecall;

    (*
     * Called before and after an assembly is unloaded.  AssemblyUnloadStarted
     * is the last point at which the AssemblyID is valid for calls to the
     * ICorProfilerInfo interface.
     * The hrStatus provides the success or failure of the operation
     *)
    procedure AssemblyUnloadStarted(assemblyId: ULONG); safecall;
    procedure AssemblyUnloadFinished(assemblyId: ULONG; hrStatus: HResult); safecall;

    (*
     *
     * MODULE EVENTS
     *
     *)

    (*
     * The CLR calls ModuleLoadStarted to notify the code
     * profiler that a module is about to be loaded.  Because this module has not
     * yet been loaded, other module information methods are not valid to
     * call in this callback.  Wait until ModuleLoadFinished() is
     * received to do things like acquire the metadata.
     *)
    procedure ModuleLoadStarted(moduleId: ULONG); safecall;

    (*
     * The CLR calls ModuleLoadFinished to notify the code
     * profiler that a module has been loaded. The hrStatus provides the
     * success or failure of the operation
     *)
    procedure ModuleLoadFinished(moduleId: ULONG; hrStatus: HResult); safecall;

    (*
     * Called before a module is being unloaded.  Use this events to collect
     * final statics that require the ModuleID to be valid.  After returning
     * from ModuleUnloadStarted, the ModuleID is no longer valid.
     * The hrStatus provides the success or failure of the operation
     *)
    procedure ModuleUnloadStarted(moduleId: ULONG); safecall;
    procedure ModuleUnloadFinished(moduleId: ULONG; hrStatus: HResult); safecall;

    (*
     * A module can get loaded through legacy means (ie: IAT or LoadLibrary) or
     * through a metadata reference.  The CLR loader therefore has many code
     * paths for determining what assembly a module lives in.  It is therefore
     * possible that after a ModuleLoadFinished event, the module does not
     * know what assembly it is in and getting the parent AssemblyID is not possible.
     * This event is fired when the module is officially attached to its parent
     * assembly.  Calling GetModuleInfo after this point will return the proper parent
     * assembly.
     *)
    procedure ModuleAttachedToAssembly(moduleId: ULONG; assemblyId: ULONG); safecall;

    (*
     *
     *  CLASS EVENTS
     *
     *)

    (*
     * The CLR calls ClassLoadStarted to notify the code profiler
     * that a class is being loaded.  The ClassID is not valid for calls to
     * the ICorProfilerInfo interface until the profiler receives a
     * ClassLoadFinished event for the same ClassID.
     *)
    procedure ClassLoadStarted(classId: ULONG); safecall;

    (*
     * The CLR calls ClassLoadFinished to notify the code
     * profiler that a class has been loaded.  The ClassID is now valid for
     * calls to the ICorProfilerInfo interface.
     * The hrStatus provides the success or failure of the operation
     *)
    procedure ClassLoadFinished(classId: ULONG; hrStatus: HResult); safecall;

    (*
     * The given class is about to be unloaded.  Use this event to gather final
     * status and clean up anything that requires the ClassID to be valid.
     * After returning from this callback the ClassID is no longer valid.
     * The hrStatus provides the success or failure of the operation
     *)
    procedure ClassUnloadStarted(classId: ULONG); safecall;
    procedure ClassUnloadFinished(classId: ULONG; hrStatus: HResult); safecall;

    (*
     *
     * JIT EVENTS
     *
     *)

    (*
     * The CLR calls FunctionUnloadStarted to notify the code
     * profiler that a function is being unloaded.  After returning from this
     * call, the FunctionID is no longer valid.
     *
     * NOTE: In the current version of the runtime, this callback never occurs.
     *)
    procedure FunctionUnloadStarted(functionId: ULONG); safecall;

    (*
     * The CLR calls JITCompilationStarted to notify the code
     * profiler that the JIT compiler is starting to compile a function.
     *
     * The fIsSafeToBlock argument tells the profiler whether or not blocking
     * will affect the operation of the runtime.  If true, blocking may cause
     * the runtime to wait for the calling thread to return from this callback.
     * Although this will not harm the runtime, it will skew the profiling
     * results.
     *
     * NOTE: It is possible to receive more than one JITCompilationStarted/
     * JITCompilationFinished pair for each method.  This is because of how
     * the runtime handles class constructors: method A starts to be JIT'd,
     * then realizes that the class ctor for class B needs to be run, so
     * JIT's it and runs it, and while it's running makes a call to original
     * method A, which causes it to be JIT'd again, and causes the original
     * (incomplete) JIT'ing of A to be aborted.  However, both attempts to
     * JIT A are reported with JIT compilation events.  If the profiler is
     * going to replace IL code for this method with SetILFunctionBody, then
     * it must do so for both JITCompilationStarted events, but may use the
     * same IL block for both.  Note that the second IL body provided through
     * SetILFunctionBody will be the IL that is used.
     *)
    procedure JITCompilationStarted(functionId: ULONG; fIsSafeToBlock: Integer); safecall;

    (*
     * The CLR calls JITCompilationFinished to notify the code
     * profiler that the JIT compiler has finished compiling a function.
     *
     * The fIsSafeToBlock argument tells the profiler whether or not blocking
     * will affect the operation of the runtime.  If true, blocking may cause
     * the runtime to wait for the calling thread to return from this callback.
     * Although this will not harm the runtime, it will skew the profiling
     * results.
     *
     * The FunctionID is now valid in ICorProfilerInfo APIs.
     *
     * The hrStatus provides the success or failure of the operation
     *)
    procedure JITCompilationFinished(functionId: ULONG; hrStatus: HResult; fIsSafeToBlock: Integer); safecall;

    (*
     * This notifies the profiler when a search for a prejitted function is
     * starting.
     *
     *    functionId: the function for which the search is being performed.
     *    bUseCachedFunction: if true, the EE uses the cached function (if applicable)
     *                        if false, the EE jits the function instead of
     *                        using a pre-jitted version.
     *)
    function JITCachedFunctionSearchStarted(functionId: ULONG): {bUseCachedFunction} Bool; safecall;

    (*
     * This notifies the profiler when a search for a cached function has been
     * performed.
     *
     *    functionId: the function for which the search has been performed.
     *    result: the result of the search.  There are two possible results:
     *        COR_PRF_CACHED_FUNCTION_FOUND
     *        COR_PRF_CACHED_FUNCTION_NOT_FOUND
     *
     *)
    procedure JITCachedFunctionSearchFinished(functionId: ULONG; AResult: COR_PRF_JIT_CACHE); safecall;

    (*
     * The CLR calls JITFunctionPitched to notify the profiler
     * that a jitted function was removed from memory.  If the pitched
     * function is called in the future, the profiler will receive new
     * JIT compilation events as it is re-jitted.
     *
     * NOTE: the FunctionID is not valid until it is re-jitted.  When it is
     * re-jitted, it will use the same FunctionID value.
     *)
    procedure JITFunctionPitched(functionId: ULONG); safecall;

    (*
     * The CLR calls JITInlining to notify the profiler that the jitter
     * is about to inline calleeId into callerId.  Set pfShouldInline to FALSE
     * to prevent the callee from being inlined into the caller, and set to
     * TRUE to allow the inline to occur.
     *
     * NOTE: Inlined functions do not provide Enter/Leave events, so if you desire
     *       an accurate callgraph, you should set FALSE.  Be aware that
     *       setting FALSE will affect performance, since inlining typically
     *       increases speed and reduces separate jitting events for the inlined
     *       method.
     *)
    function JITInlining(callerId: ULONG; calleeId: ULONG): {fShouldInline} Bool; safecall;

    (*
     *
     * THREAD EVENTS
     *
     *)

    (*
     * The CLR calls ThreadCreated to notify the code profiler
     * that a thread has been created.  The ThreadID is valid immediately.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure ThreadCreated(threadId: ULONG); safecall;

    (*
     * The CLR calls ThreadDestroyed to notify the code profiler
     * that a thread has been destroyed.  The ThreadID is no longer valid.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure ThreadDestroyed(threadId: ULONG); safecall;

    (*
     * The CLR calls ThreadAssignedToOSThread to tell the profiler
     * that a managed thread is being implemented via a particualr OS thread.
     * This callback exists so that the profiler can maintain an accurate
     * OS to Managed thread mapping across fibres.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure ThreadAssignedToOSThread(managedThreadId: ULONG; osThreadId: ULONG); safecall;

    (*
     *
     * REMOTING EVENTS
     *
     *
     * NOTE: Inprocess debugging not supported from these callbacks.
     *)

    //
    // Client-side events
    //

    (*
     * NOTE: each of the following pairs of callbacks will occur on the same
     *       thread
     *   RemotingClientInvocationStarted  & RemotingClientSendingMessage
     *   RemotingClientReceivingReply     & RemotingClientInvocationFinished
     *   RemotingServerInvocationReturned & RemotingServerSendingReply

    (*
     * The CLR calls RemotingClientInvocationStarted to notify the profiler that
     * a remoting call has begun.  This event is the same for synchronous and
     * asynchronous calls.
     *)
    procedure RemotingClientInvocationStarted; safecall;

    (*
     * The CLR calls RemotingClientSendingMessage to notify the profiler that
     * a remoting call is requiring the the caller to send an invocation request through
     * a remoting channel.
     *
     * pCookie  - if remoting GUID cookies are active, this value will correspond with the
     *            the value provided in RemotingServerReceivingMessage, if the channel
     *            succeeds in transmitting the message, and if GUID cookies are active on
     *            the server-side process.  This allows easy pairing of remoting calls,
     *            and the creation of a logical call stack.
     * fIsAsync - is true if the call is asynchronous.
     *)
    procedure RemotingClientSendingMessage(const pCookie: TGuid; fIsAsync: Bool); safecall;

    (*
     * The CLR calls RemotingClientReceivingReply to notify the profiler that
     * the server-side portion of a remoting call has completed and that the client is
     * now receiving and about to process the reply.
     *
     * pCookie  - if remoting GUID cookies are active, this value will correspond with the
     *            the value provided in RemotingServerSendingReply, if the channel
     *            succeeds in transmitting the message, and if GUID cookies are active on
     *            the server-side process.  This allows easy pairing of remoting calls.
     * fIsAsync - is true if the call is asynchronous.
     *)
    procedure RemotingClientReceivingReply(const pCookie: TGuid; fIsAsync: Bool); safecall;

    (*
     * The CLR calls RemotingClientInvocationFinished to notify the profiler that
     * a remoting invocation has run to completion on the client side.  If the call was
     * synchronous, this means that it has also run to completion on the server side.  If
     * the call was asynchronous, a reply may still be expected when the call is handled.
     * If the call is asynchronous, and a reply is expected, then the reply will occur in
     * the form of a call to RemotingClientReceivingReply and an additional call to
     * RemotingClientInvocationFinished to indicate the required secondary processing of
     * an asynchronous call.
     *)
    procedure RemotingClientInvocationFinished; safecall;

    //
    // Server-side events
    //

    (*
     * The CLR calls RemotingServerReceivingMessage to notify the profiler that
     * the process has received a remote method invocation (or activation) request.  If
     * the message request is asynchronous, then the request may be serviced by any
     * arbitrary thread.
     *
     * pCookie  - if remoting GUID cookies are active, this value will correspond with the
     *            the value provided in RemotingClientSendingMessage, if the channel
     *            succeeds in transmitting the message, and if GUID cookies are active on
     *            the client-side process.  This allows easy pairing of remoting calls.
     * fIsAsync - is true if the call is asynchronous.
     *)
    procedure RemotingServerReceivingMessage(const pCookie: TGuid; fIsAsync: Bool); safecall;

    (*
     * The CLR calls RemotingServerInvocationStarted to notify the profiler that
     * the process is invoking a method due to a remote method invocation request.
     *)
    procedure RemotingServerInvocationStarted; safecall;

    (*
     * The CLR calls RemotingServerInvocationReturned to notify the profiler that
     * the process has finished invoking a method due to a remote method invocation request.
     *)
    procedure RemotingServerInvocationReturned; safecall;

    (*
     * The CLR calls RemotingServerSendingReply to notify the profiler that
     * the process has finished processing a remote method invocation request and is
     * about to transmit the reply through a channel.
     *
     * pCookie  - if remoting GUID cookies are active, this value will correspond with the
     *            the value provided in RemotingClientReceivingReply, if the channel
     *            succeeds in transmitting the message, and if GUID cookies are active on
     *            the client-side process.  This allows easy pairing of remoting calls.
     * fIsAsync - is true if the call is asynchronous.
     *)
    procedure RemotingServerSendingReply(const pCookie: TGuid; fIsAsync: Bool); safecall;

    (*
     *
     * TRANSITION EVENTS
     *
     *)

    (*
     * The CLR calls UnmanagedToManagedTransition to notify the
     * code profiler that a transition from unmanaged code to managed code has
     * occurred. functionId is always the ID of the callee, and reason
     * indicates whether the transition was due to a call into managed code from
     * unmanaged, or a return from an unmanaged function called by a managed one.
     *
     * Note that if the reason is COR_PRF_TRANSITION_RETURN, then the functionId
     * is that of the unmanaged function, and will never have been jitted.
     * Unmanaged functions still have some basic information associated with
     * them, such as a name, and some metadata.
     *
     * Note that if the reason is COR_PRF_TRANSITION_RETURN and the callee was
     * a PInvoke call indirect, then the runtime does not know the destination
     * of the call and functionId will be NULL.
     *
     * Note that if the reason is COR_PRF_TRANSITION_CALL then it may be possible
     * that the callee has not yet been JIT-compiled.
     *)
    procedure UnmanagedToManagedTransition(functionId: ULONG; reason: COR_PRF_TRANSITION_REASON); safecall;

    (*
     * The CLR calls ManagedToUnmanagedTransition to notify the
     * code profiler that a transition from managed code to unmanaged code has
     * occurred. functionId is always the ID of the callee, and reason
     * indicates whether the transition was due to a call into unmanaged code from
     * managed, or a return from an managed function called by an unmanaged one.
     *
     * Note that if the reason is COR_PRF_TRANSITION_CALL, then the functionId
     * is that of the unmanaged function, and will never have been jitted.
     * Unmanaged functions still have some basic information associated with
     * them, such as a name, and some metadata.
     *
     * Note that if the reason is COR_PRF_TRANSITION_CALL and the callee is
     * a PInvoke call indirect, then the runtime does not know the destination
     * of the call and functionId will be NULL.
     *)
    procedure ManagedToUnmanagedTransition(functionId: ULONG; reason: COR_PRF_TRANSITION_REASON); safecall;

    (*
     *
     * RUNTIME SUSPENSION EVENTS
     *
     *)

    (*
     * NOTE: None of the suspension callbacks are supported by in-process
     *       debugging *except* ObjectReferences.
     *)

    (*
     * The CLR calls RuntimeSuspendStarted to notify the code profiler
     * that the runtime is about to suspend all of the runtime threads.
     * All runtime threads that are in unmanaged code are permitted to continue
     * running until they try to re-enter the runtime, at which point they will
     * also suspend until the runtime resumes.  This also applies to new threads
     * that enter the runtime.  All threads within the runtime are either
     * suspended immediately if they are in interruptible code, or asked to
     * suspend when they do reach interruptible code.
     *
     * suspendReason make be any of the following values:
     *  COR_PRF_SUSPEND_FOR_GC
     *      the runtime is suspending to service a GC request.  The GC-related
     *      callbacks will occur between the RuntimeSuspendFinished and
     *      RuntimeResumeStarted events.
     *  COR_PRF_SUSPEND_FOR_CODE_PITCHING
     *      the runtime is suspending so that code pitching may occur.  This
     *      only occurs when the EJit is active with code pitching enabled.
     *      Code pitching callbacks will occur between the
     *      RuntimeSuspendFinished and RuntimeResumeStarted events.
     *  COR_PRF_SUSPEND_FOR_APPDOMAIN_SHUTDOWN
     *      the runtime is suspending so that an AppDomain can be shut down.
     *      While the runtime is suspended, the runtime will determine which
     *      threads are in the AppDomain that is being shut down, set them to
     *      abort when they resume, and then resumes the runtime.  There are
     *      no AppDomain-specific callbacks during this suspension.
     *  COR_PRF_SUSPEND_FOR_SHUTDOWN
     *      the runtime is shutting down, and it must suspend all threads to
     *      complete the operation.
     *  COR_PRF_SUSPEND_FOR_GC_PREP
     *      the runtime is preparing for a GC.
     *  COR_PRF_SUSPEND_FOR_INPROC_DEBUGGER
     *      the runtime is suspending for in-process debugging.
     *  COR_PRF_SUSPEND_OTHER
     *      the runtime is suspending for a reason other than those above.
     *)
    procedure RuntimeSuspendStarted(suspendReason: COR_PRF_SUSPEND_REASON); safecall;

    (*
     * The CLR calls SyncForSuspendFinished to notify the code profiler
     * that the runtime has suspended all threads needed for a runtime
     * suspension.  Note that not all runtime threads are required to be
     * suspended, as described in the comment for SyncForSuspendStarted.
     *
     * NOTE: It is guaranteed that this event will occur on the same ThreadID
     * as RuntimeSuspendStarted occurred on.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure RuntimeSuspendFinished; safecall;

    (*
     * The CLR calls RuntimeSuspendAborted to notify the code profiler
     * that the runtime is aborting the runtime suspension that was occurring.
     * This may occur if two threads simultaneously attempt to suspend the
     * runtime.
     *
     * NOTE: It is guaranteed that this event will occur on the same ThreadID
     * as the RuntimeSuspendStarted occurred on, and that only one of
     * RuntimeSuspendFinished and RuntimeSuspendAborted may occur on a single
     * thread following a RuntimeSuspendStarted event.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure RuntimeSuspendAborted; safecall;

    (*
     * The CLR calls RuntimeResumeStarted to notify the code profiler
     * that the runtime is about to resume all of the runtime threads.
     *
     * NOTE: If a thread successfully suspended the runtime, then it is *NOT*
     * guaranteed that the call to RuntimeResumeStarted will be called on the
     * same ThreadID.  In this case, it is also not guaranteed that the thread
     * calling RuntimeResumeStarted will have a non-zero ThreadID.
     * However, if a thread unsuccessfully suspends the runtime
     * (i.e., there was a conflict with another suspension), then it *IS*
     * guaranteed that the calls to RuntimeSuspspendStarted,
     * RuntimeSuspendAborted and RuntimeResumeStarted will be called from the
     * same ThreadID.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure RuntimeResumeStarted; safecall;

    (*
     * The CLR calls RuntimeResumeFinished to notify the code profiler
     * that the runtime has finished resuming all of it's threads and is now
     * back in normal operation.
     *
     * NOTE: It is *NOT* guaranteed that this event will occur on the same
     * ThreadID as the RuntimeSuspendStarted occurred on, but is guaranteed
     * to occur on the same ThreadID as the RuntimeResumeStarted occurred on.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure RuntimeResumeFinished; safecall;

    (*
     * The CLR calls ThreadSuspended to notify the code profiler
     * that a particular thread has been suspended.  All threads within managed
     * code must be suspended.  If a thread is in unmanaged code, it will be
     * allowed to continue, but will suspend upon re-entering the runtime
     * and will fire this event.  Thus, this notification could occur after
     * a suspension has completed, but before the runtime resumes.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure RuntimeThreadSuspended(threadId: ULONG); safecall;

    (*
     * The CLR calls ThreadResumed to notify the code profiler
     * that a particular thread has been resumed after being suspended due to
     * a runtime suspension.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure RuntimeThreadResumed(threadId: ULONG); safecall;

    (*
     *
     * GC EVENTS
     *
     *)

    (*
     * The CLR calls MovedReferences with information about
     * object references that moved as a result of garbage collection.
     *
     * cMovedObjectIDRanges is a count of the number of ObjectID ranges that
     *      were moved.
     * oldObjectIDRangeStart is an array of elements, each of which is the start
     *      value of a range of ObjectID values before being moved.
     * newObjectIDRangeStart is an array of elements, each of which is the start
     *      value of a range of ObjectID values after being moved.
     * cObjectIDRangeLength is an array of elements, each of which states the
     *      size of the moved ObjectID value range.
     *
     * The last three arguments of this function are parallel arrays.
     *
     * In other words, if an ObjectID value lies within the range
     *      oldObjectIDRangeStart[i] <= ObjectID < oldObjectIDRangeStart[i] + cObjectIDRangeLength[i] 
     * for 0 <= i < cMovedObjectIDRanges, then the ObjectID value has changed to
     *      ObjectID - oldObjectIDRangeStart[i] + newObjectIDRangeStart[i] 
     *
     * NOTE: All of these callbacks are made while the runtime is suspended, so
     *       none of the ObjectID values can change until the runtime resumes
     *       and another GC occurs.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *
     * NOTE: The profiler will receive GC-related events (except ObjectAllocated)
     *       when the profiler has been suspended for COR_PRF_SUSPEND_FOR_GC *except*
     *       for one special case.  When the runtime is shutting down, there is a
     *       stage where it is suspended for COR_PRF_SUSPEND_FOR_SHUTDOWN reason and
     *       is never resumed.  But after this suspension a garbage collection may
     *       occur without a COR_PRF_SUSPEND_FOR_GC suspension notification, and
     *       the profiler will thus receive GC-related callbacks.
     *)
    procedure MovedReferences(cMovedObjectIDRanges: ULONG;
                              var oldObjectIDRangeStart, newObjectIDRangeStart,
                                  cObjectIDRangeLength: TULONGArray); safecall;

    (*
     * The CLR calls ObjectAllocated to notify the code profiler
     * an object was allocated on the heap.
     *)
    procedure ObjectAllocated(objectId, classId: ULONG); safecall;

    (*
     * The CLR calls ObjectsAllocatedByClass to notify the code
     * profiler about the number of objects of a particular class that were
     * allocated since the previous garbage collection. The classes and the
     * counts are passed in parallel arrays.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *
     * NOTE: This callback will not report objects allocated in the large
     *       object heap.
     *)
    procedure ObjectsAllocatedByClass(cClassCount: ULONG;
                              var classIds, cObjects: TULONGArray); safecall;

    (*
     * The CLR calls ObjectReferences to provide information
     * about objects in memory referenced by a given object.  This function
     * is called for each object remaining in the GC heap after a collection
     * has completed.  If the profiler returns an error from this callback,
     * the profiling services will discontinue invoking this callback until the
     * next GC.  This callback can be used in conjunction with the
     * RootReferences callback to create a complete object reference graph for
     * the runtime.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure ObjectReferences(objectId: ULONG; classId: ULONG; cObjectRefs: ULONG;
                              var objectRefIds: TULONGArray); safecall;

    (*
     * The CLR calls RootReferences with information about root
     * references after a garbage collection has occurred. Static object
     * references and references to objects on a stack are co-mingled in the
     * arrays.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure RootReferences(cRootRefs: ULONG; var rootRefIds: TULONGArray); safecall;

    (*
     *
     * EXCEPTION EVENTS
     *
     *
     * NOTE: Inprocess debugging not supported from any exception callbacks
     *       except ExceptionThrown.
     *)

    //
    // Exception creation
    //

    (*
     * The CLR calls ExceptionThrown to notify the code
     * profiler that an exception has been thrown.
     *
     * NOTE: This function is only called if the CLR exception handler
     *       is called to process an exception.
     *
     * NOTE: For in-process debugging, thrownObjectId is always NULL.  The
     *       actual thrown object can be found through
     *       ICorDebugThread::GetCurrentException
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *)
    procedure ExceptionThrown(thrownObjectId: ULONG); safecall;

    //
    // Search phase
    //

    (*
     * The CLR calls ExceptionSearchFunctionEnter to notify the profiler
     * that the search phase of exception handling has entered a function.
     *)
    procedure ExceptionSearchFunctionEnter(functionId: ULONG); safecall;

    (*
     * The CLR calls ExceptionSearchFunctionLeave to notify the profiler
     * that the search phase of exception handling has left a function.
     *)
    procedure ExceptionSearchFunctionLeave; safecall;

    (*
     * The CLR will call ExceptionSearchFilterEnter just before excecuting
     * a user filter.  The functionID is that of the function containing the filter.
     *)
    procedure ExceptionSearchFilterEnter(functionId: ULONG); safecall;

    (*
     * The CLR will call ExceptionSearchFilterLeave immediately after
     * executing a user filter.
     *)
    procedure ExceptionSearchFilterLeave; safecall;

    (*
     * The CLR will call ExceptionSearchCatcherFound when the search
     * phase of exception handling has located a handler for the exception that
     * was thrown.
     *)
    procedure ExceptionSearchCatcherFound(functionId: ULONG); safecall;

    (*
     * The CLR calls ExceptionHandlerEnter when the runtime's
     * [Win32 SEH] exception handler is entered.  There may or may not be
     * managed functions guarded by this handler, the profiler may or may not
     * receive exception-related callbacks between ExceptionOSHandlerEnter and
     * ExceptionOSHandlerLeave callbacks.
     *
     * This notification is provided to allow profilers to detect unmanaged-to-
     * managed transitions in stack searches and unwinds.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *
     * NOTE: NOT YET IMPLEMENTED.
     *)
    procedure ExceptionOSHandlerEnter(__unused: ULONG); safecall;

    (*
     * This function is similar to ExceptionHandlerEnter, except that
     * it is called just before the runtime's exception handler returns.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *
     * NOTE: NOT YET IMPLEMENTED.
     *)
    procedure ExceptionOSHandlerLeave(__unused: ULONG); safecall;

    //
    // Unwind phase
    //

    (*
     * The CLR calls ExceptionUnwindFunctionEnter to notify the profiler
     * that the unwind phase of exception handling has entered a function.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *)
    procedure ExceptionUnwindFunctionEnter(functionId: ULONG); safecall;

    (*
     * The CLR calls ExceptionUnwindFunctionLeave to notify the profiler
     * that the unwind phase of exception handling has left a function.  The
     * function instance and it's stack data has now been removed from the stack.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *)
    procedure ExceptionUnwindFunctionLeave; safecall;

    (*
     * The CLR calls ExceptionUnwindFinallyEnter to notify the profiler
     * that the unwind phase of exception is entering a finally clause contained
     * in the specified function.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *)
    procedure ExceptionUnwindFinallyEnter(functionId: ULONG); safecall;

    (*
     * The CLR calls ExceptionUnwindFinallyLeave to notify the profiler
     * that the unwind phase of exception is leaving a finally clause.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *)
    procedure ExceptionUnwindFinallyLeave; safecall;

    (*
     * The CLR calls this function just before passing control to 
     * the appropriate catch block.  Note that this is called only if the 
     * catch point is in JIT'ed code.  An exception that is caught in 
     * unmanaged code, or in the internal code of the CLR will
     * not generate this notification.  The ObjectID is passed again since
     * a GC could have moved the object since the ExceptionThrown
     * notification.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *)
    procedure ExceptionCatcherEnter(functionId: ULONG; objectId: ULONG); safecall;

    (*
     * The CLR calls ExceptionCatcherLeave when the runtime leaves
     * the catcher's code.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *)
    procedure ExceptionCatcherLeave; safecall;

    (*
     *  CLR<->COM interop vtable creation/destruction.
     *)

    (*
     * The CLR calls this function when an CLR<->COM interop vtable
     * for a particular IID and for a particular class has been created.
     * This provides the ClassID of the class for which this vtable has been
     * created, the IID it implements, the start of the vtable and how many
     * slots are in it.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *
     * NOTE: It is possible to receive a NULL GUID if the interface is
     *       internal only.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure COMClassicVTableCreated(wrappedClassId: ULONG; const implementedIID: TGuid;
                                     pVTable: Pointer; cSlots: LongWord); safecall;

    (*
     * The CLR calls this function when a CLR<->COM interop vtable is
     * being destroyed.  Provided are the ClassID, IID and vtable pointer for
     * the destroyed vtable.
     *
     * NOTE: The profiler should not block here, since the stack may not be in a
     *       GC-friendly state and so preemptive GC cannot be enabled.  If the
     *       profiler blocks here and a GC is attempted, the runtime will block
     *       until this callback returns.  Also, the profiler may NOT call into
     *       managed code or in any way cause a managed memory allocation.
     *
     * NOTE: It is possible to receive a NULL GUID if the interface is
     *       internal only.
     *
     * NOTE: Inprocess debugging not supported from this callback.
     *)
    procedure COMClassicVTableDestroyed(wrappedClassId: ULONG; const implementedIID: TGuid;
                                       pVTable: Pointer); safecall;

    (*
     * When the CLR itself catches an exception, these functions are called
     * to notify the profiler.  Between the Found and Execute callbacks the
     * CLR may also unwind managed frames and so normal ExceptionUnwind*
     * functions can be called in between.
     *
     * NOTE: for backward compatibility the profiler must also set the
     *       COR_PRF_MONITOR_CLR_EXCEPTIONS event flag to receive these events.
     *
     * NOTE: Cannot use inprocess debugging from these callbacks.
     *)
    procedure ExceptionCLRCatcherFound; safecall;
    procedure ExceptionCLRCatcherExecute; safecall;
  end;

(*
 * The CLR implements the ICorProfilerInfo interface. This interface is
 * used by a code profiler to communicate with the CLR to control event
 * monitoring and request information. The CLR passes an
 * ICorProfilerInfo interface to each code profiler during initialization.
 *
 * A code profiler can call methods on the ICorProfilerInfo interface to get
 * information about managed code being executed under the control of the COM+
 * Runtime.
 *
 * The ICorProfilerInfo interface implemented by the CLR uses the free
 * threaded model. Events are dispatched from within the CLR or on a
 * thread that is making the code profiler method call. Interface methods
 * implemented by the CLR can be called from any thread (that has been
 * CoInitialized) at any time.
 *
 * The methods implemented on this interface return S_OK on success, or E_FAIL
 * on failure.
 *
 * For parameters returned from these methods that are [out] buffer parameters
 * filled with pointers to buffers (and not NULL pointers), these objects were
 * allocated by the runtime using CoTaskMemAlloc. The code profiler must free
 * these objects using calls to CoTaskMemFree.
 *)

  ICorProfilerInfo = interface(IUnknown)
    ['{28B5557D-3F3F-48b4-90B2-5F9EEA2F6C48}']
    (*
     * The code profiler calls GetClassFromObject to obtain the ClassID of an
     * object given its ObjectID.
     *)
    function GetClassFromObject(objectId: ULONG): {pClassId} ULONG; safecall;

    (*
     * The code profiler calls GetClassFromToken to obtain the ClassID of a
     * class given its metadata.
     *)
    function GetClassFromToken(moduleId: ULONG; typeDef: ULONG): {pClassId} ULONG; safecall;

    (*
     * The code profiler calls GetCodeInfo to obtain information about a
     * JIT-compiled function.
     *
     * An error will be returned if GetCodeInfo is called with a FunctionID
     * for a function that has not been JIT-compiled.
     *)
    procedure GetCodeInfo(functionId: ULONG; out pStart: PByte; out pcSize: ULONG); safecall;

    (*
     * The code profiler calls GetEventMask to obtain the current event
     * categories for which it is to receive event notifications from the COM+
     * Runtime.
     *)
    function GetEventMask: {pdwEvents} ULONG; safecall;

    (*
     * The code profiler calls GetFunctionFromIP to map an instruction pointer
     * in managed code to a FunctionID.
     *)
    function GetFunctionFromIP(var ip: Byte): {pFunctionId} ULONG; safecall;

    (*
     * The code profiler calls GetFunctionFromToken to obtain the FunctionID of
     * a function given its metadata.
     *)
    function GetFunctionFromToken(moduleId: ULONG; token: ULONG): {pFunctionId} ULONG; safecall;

    (*
     * The code profiler calls GetHandleFromThread to map a ThreadID to a Win32
     * thread handle.
     *)
    function GetHandleFromThread(threadId: ULONG): {phThread} THandle; safecall;

    (*
     * The code profiler calls GetObjectSize to obtain the size of an object.
     *)
    function GetObjectSize(objectId: ULONG): {pcSize} ULONG; safecall;

    (*
     * This will return S_OK if the ClassID provided is an array class, and will
     * fill out the information for any non-null out params.  S_FALSE will be
     * returned if the ClassID is not an array.
     *
     * classId       : the ClassID to return information about
     * pBaseElemType : the array's base element type
     * pBaseClassId  : the base ClassID if the element type == ELEMENT_TYPE_CLASS
     * pcRank        : the number of dimensions of the array
     *)
    procedure IsArrayClass(classId: ULONG; out pBaseElemType: ULONG; 
                           out pBaseClassId: ULONG; out pcRank: ULONG); safecall;

    (*
     * The code profiler calls GetThreadInfo to obtain the Win32 thread ID for
     * the specified thread.
     *)
    function GetThreadInfo(threadId: ULONG): {pdwWin32ThreadId} ULONG; safecall;

    (*
     * The code profiler calls GetCurrentThreadID to get the managed thread ID
     * for the current thread.
     *)
    function GetCurrentThreadID: {pThreadId} ULONG; safecall;

    (*
     * Returns the parent module a class is defined in, along with the
     * metadata token for the class.  One can hook the ModuleLoadStarted
     * event to obtain the metadata interface for a given module.  The token
     * can then be used to access the metadata for this class.
     *)
    procedure GetClassIDInfo(classId: ULONG; out pModuleId: ULONG; out pTypeDefToken: ULONG); safecall;

    (*
     * Return the parent class for a given function.  Also return the metadata
     * token which can be used to read the metadata.
     *)
    procedure GetFunctionInfo(functionId: ULONG; out pClassId: ULONG; out pModuleId: ULONG; 
                              out pToken: ULONG); safecall;

    (*
     * The code profiler calls SetEventMask to sets the event categories for
     * which it is set to receive notification from the CLR.
     *)
    procedure SetEventMask(dwEvents: ULONG); safecall;

    (*
     * The code profiler calls SetFunctionHooks to specify its own callback
     * replacements for ICorProfilerCallback::FunctionEntry,
     * ICorProfilerCallback::FunctionExit and
     * ICorProfilerCallback::FunctionTailcall
     *)
    procedure SetEnterLeaveFunctionHooks(pFuncEnter: FunctionEnter;
                                         FunctionLeave: FunctionLeave;
                                         FunctionTailcall: FunctionTailCall); safecall;

    (*
     * This is used for mapping FunctionIDs to alternative values that will be
     * passed to the callbacks
     *)
    procedure SetFunctionIDMapper(var pFunc: FunctionIDMapper); safecall;

    (*
     * For a given function, retrieve the token value and an instance of the
     * meta data interface which can be used against this token.
     *)
    procedure GetTokenAndMetaDataFromFunction(functionId: ULONG; const riid: TGuid;
                                              out ppImport: IUnknown; out pToken: ULONG); safecall;

    (*
     * Retrieve information about a given module.
     *)
    procedure GetModuleInfo(moduleId: ULONG; ppBaseLoadAddress: PPByte; cchName: ULONG;
                            pcchName: PULONG; szName: PWideChar;
                            pAssemblyId: PULONG); safecall;

    (*
     * Get a metadata interface instance which maps to the given module.
     * One may ask for the metadata to be opened in read+write mode, but
     * this will result in slower metadata execution of the program, because
     * changes made to the metadata cannot be optimized as they were from
     * the compiler.
     *
     * NOTE: the only values valid for dwOpenFlags are ofRead and ofWrite.
     *)
    function GetModuleMetaData(moduleId: ULONG; dwOpenFlags: ULONG;
                                const riid: TGuid): {ppOut} IUnknown; safecall;

    (*
     * Retrieve a pointer to the body of a method starting at it's header.
     * A method is coped by the module it lives in.  Because this function
     * is designed to give a tool access to IL before it has been loaded
     * by the Runtime, it uses the metadata token of the method to find
     * the instance desired.  Note that this function has no effect on
     * already compiled code.
     *
     * GetILFunctionBody can return CORPROF_E_FUNCTION_NOT_IL if the methodId
     * points to a method without any IL (such as an abstract method, or a
     * P/Invoke method).
     *)
    procedure GetILFunctionBody(moduleId: ULONG; methodId: ULONG;
                                out ppMethodHeader: PByte;
                                out pcbMethodSize: ULONG); safecall;

    (*
     * IL method bodies must be located as RVA's to the loaded module, which
     * means they come after the module within 4 gb.  In order to make it
     * easier for a tool to swap out the body of a method, this allocator
     * will ensure memory is allocated after that point.
     *)
    function GetILFunctionBodyAllocator(moduleId: ULONG): {ppMalloc} IMethodMalloc; safecall;

    (*
     * Replaces the method body for a function in a module.  This will replace
     * the RVA of the method in the metadata to point to this new method body,
     * and adjust any internal data structures as required.  This function can
     * only be called on those methods which have never been compiled by a JITTER.
     * Please use the GetILFunctionAllocator to allocate space for the new method to
     * ensure the buffer is compatible.
     *)
    procedure SetILFunctionBody(moduleId: ULONG; methodId: ULONG;
                                var pbNewILMethodHeader: Byte); safecall;

    (*
     * Retrieve app domain information given its id.
     *)
    procedure GetAppDomainInfo(appDomainId: ULONG; cchName: ULONG; out pcchName: ULONG; 
                               szName: PWideChar; out pProcessId: ULONG); safecall;

    (*
     * Retrieve information about an assembly given its ID.
     *)
    procedure GetAssemblyInfo(assemblyId: ULONG; cchName: ULONG; out pcchName: ULONG;
                              szName: PWideChar; out pAppDomainId: ULONG;
                              out pModuleId: ULONG); safecall;

    (*
     * Marks a function as requiring a re-JIT.  The function will be re-JITted
     * at its next invocation.  The normal profiller events will give the profiller
     * an opportunity to replace the IL prior to the JIT.  By this means, a tool
     * can effectively replace a function at runtime.  Note that active instances
     * of the function are not affected by the replacement.
     *
     * NOTE: memory used by the current jitted code for this method will not be
     *       released, as it may currently be in use by one or more threads and
     *       may have associated data on the stack.
     *)
    procedure SetFunctionReJIT(functionId: ULONG); safecall;

    (*
     * ForceGC forces a GC to occur within the runtime.
     *
     * NOTE: This method needs to be called from a thread that does not have any
     * profiler callbacks on its stack. The most convenient way to implement this is
     * to create a separate thread within the profiler and have it call ForceGC when
     * signalled.
     *)
    procedure ForceGC; safecall;

    (*
     * fStartJit should be set to true to indicate that the invocation is in advance
     *      of JITting the function.  If this is merely to say that the
     *      function's map is changing, it should be false.
     *
     * The format of the map is as follows:
     *      The debugger will assume that each oldOffset refers to an IL offset
     *  within the original, unmodified IL code.  newOffset refers to the corresponding
     *  IL offset within the new, instrumented code.
     *      If the offset of the original IL is exactly equal to an oldOffset, then
     *  it's new offset within the instrumented code is given by newOffset.  If the
     *  original offset isn't exactly equal, then the new offset is equal to
     *
     *      rgILMapEntries[i] .newOffset - rgILMapEntries[i] .oldOffset + original offset
     *
     *  where
     *
     *      0 <= i < cILMapEntries, and rgILMapEntries[i] .oldOffset < original offset,
     *  and there does not exist a j  such that
     *
     *      rgILMapEntries[i] .oldOffset < rgILMapEntries[j] .oldOffset < original offset
     *)
    procedure SetILInstrumentedCodeMap(functionId: ULONG; fStartJit: Integer; 
                                       cILMapEntries: ULONG;
                                       var rgILMapEntries: COR_IL_MAP); safecall;

    (*
     * GetInprocInspectionInterface is used to get an interface to the
     * in-process portion of the debug interface, which is useful for things
     * like doing a stack trace.  It is expected that the returned interface
     * will be queried for ICorDebug.
     *
     * ppicd: *ppicd will be filled in with a pointer to the interface, or
     *          NULL if the interface is unavailable.
     *)
    function GetInprocInspectionInterface: {ppicd} IUnknown; safecall;

    (*
     * GetInprocInspectionIThisThread is used to get an interface to the
     * in-process portion of the debug interface, that is specific to
     * this thread.  It's expected that the returned interface will be
     * be queried for ICorDebugThread, which can then be used to immediately
     * do a stack trace.
     *
     * ppicd: *ppicd will be filled in with a pointer to the interface, or
     *          NULL if the interface is unavailable.
     *)
    function GetInprocInspectionIThisThread: {ppicd} IUnknown; safecall;

    (*
     * This will return the ContextID currently associated with the calling
     * runtime thread.  This will set pContextId to NULL if the calling thread
     * is not a runtime thread.
     *)
    function GetThreadContext(threadId: ULONG): {pContextId} ULONG; safecall;

    (*
     * The profiler MUST call this function before using the in-process debugging
     * APIs.  fThisThreadOnly indicates whether in-proc debugging will be used to
     * trace the stack of the current managed thread only, or whether it might be
     * used to trace the stack of any managed thread.
     *
     * The pdwProfilerContext argument is information that the profiler must save
     * so that it may use it during EndInprocDebugging.  The profiler must pass
     * the value returned by this API to EndInprocDebugging in an unaltered
     * state.
     *)
    function BeginInprocDebugging(fThisThreadOnly: Integer): {pdwProfilerContext} ULONG; safecall;

    (*
     * The profiler MUST call this function when it is done using the in-process
     * debugging APIs.  Failing to do so will result in undefined behaviour of
     * the runtime.
     *
     * The dwProfilerContext is the value of the [out] param pdwProfilerContext
     * returned by BeginInprocDebugging.
     *)
    procedure EndInprocDebugging(dwProfilerContext: ULONG); safecall;

    (*
     * GetILToNativeMapping returns a map from IL offsets to native
     * offsets for this code. An array of COR_PROF_IL_TO_NATIVE_MAP
     * structs will be returned, and some of the ilOffsets in this array
     * may be the values specified in CorProfIlToNativeMappingTypes.
     *)
    procedure GetILToNativeMapping(functionId: ULONG; cMap: ULONG; out pcMap: ULONG;
                                   out map: PCOR_DEBUG_IL_TO_NATIVE_MAP); safecall;
  end;

(*
 * This is simple allocator that only allows you to allocate memory.
 * You may not free it.  This should be used in conjunction with
 * SetILFunctionBody.
 *)
  IMethodMalloc = interface(IUnknown)
    ['{A0EFB28B-6EE2-4d7b-B983-A75EF7BEEDB8}']
    (*
     * Tries to allocate memory above the start address of the module from
     * which it was created.  It is important to note that this method may
     * fail to allocate the memory specified above the start address, and
     * may as a result return E_OUTOFMEMORY.
     *)
    function Alloc(cb: ULONG): Pointer; stdcall;
  end;

implementation

end.
